import processing.core.*; 
import processing.xml.*; 

import treemap.*; 
import java.util.Calendar; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class sketch_2dspatial extends PApplet {




Treemap map;
MapLayout layoutAlgorithm = new SquarifiedLayout();

int maxFontSize = 100;
int minFontSize = 10;

PFont font, font2;

int[] frequency = new int[800];
int totalCheckouts;

String dataFile = "query/700s_2012-13.csv"; 

String[] deweyClassNames = {"Arts","Civic Landscape Art", "Architecture", "Plastic Arts; Sculpture", "Drawing & Decorative Arts","Painting & Paintings","Graphic Arts; Printmaking",
                            "Photography","Music","Recreational & performing arts"};

int[][] palette = {{0xffDA4DFF, 0xffCB00FF, 0xff5F276E, 0xffA200CC, 0xff650080},
                     {0xff1AFF7A, 0xff00A847, 0xff00692C, 0xffFF4134, 0xff730800},
                     {0xff4CCCFF, 0xff00B6FF, 0xff0092CC, 0xff26667F, 0xff005B7F},
                   };

public void setup(){
  size(1000,800);
  background(255);
  smooth();
  DeweyMap mapData = new DeweyMap();
  font = createFont("Helvetica",10,true);
  font2 = createFont("Helvetica",18,true);
 
  int[] data;
  String[] foo;
// ------------- read in Data ------------- \\ 

      String[] datum = loadStrings(dataFile);
      data = new int[datum.length];
      foo = new String[datum.length];
      for (int i = 1; i < datum.length; ++i){
         String results[] = split(datum[i],",");
         results[2] = results[2].substring(0,3);
         foo[i] = results[2];
         int a = (int)Integer.parseInt(results[0]);  //year
         int b = (int)Integer.parseInt(results[1]);  //month
         int val = (int)Float.parseFloat(results[2]);//deweyNumber
         data[i] = val;     
      }

// ------------- Create the Map ------------- \\ 
  data = sort(data);
  for (int i = 0; i < data.length; i++){
      mapData.addDewey(data[i]);
  }
  
  mapData.finishAdd();
  map = new Treemap(mapData, 0, 1.0f, width, height); 
}

public void draw(){
  background(255);
  map.setLayout(layoutAlgorithm);
  map.updateLayout();
  map.draw();
  noLoop();
  drawStaticText();
  totalCheckouts = 0;
  for (int i = 0; i < frequency.length; i++){
      totalCheckouts += frequency[i];
      if (frequency[i] > 0) {
        println("The Number of checkouts for Dewey Class " + i + " is " + frequency[i]);
      } 
  }
  
  println(totalCheckouts);
}


// Code is based on the treemap class from Visualizing Data, First Edition, Copyright 2008 Ben Fry.

                            
class DataItem extends SimpleMapItem {
  int deweyClass;
  int margin = 3;

  DataItem(int deweyClass) {
    this.deweyClass = deweyClass;
  }
  
  public void draw() {
    
    strokeWeight(1);
  // noStroke();
  // smooth();
    fill(palette[2][(int)random(0,palette[2].length)]);
    
    rect(x, y, w, h);
   
    for (int i = minFontSize; i <= maxFontSize; i++) {
      textFont(font,i);
     String s = ""+deweyClass;
      if (w < textWidth(s) + margin || h < (textAscent()+textDescent()) + margin) {
        textFont(font,i);
        break;
      }
    }

    // text
    fill(0);
    textAlign(CENTER, CENTER);
    text(deweyClass, x + w/2, y + h/2);
  }
}



// Code is based on the treemap class from Visualizing Data, First Edition, Copyright 2008 Ben Fry.

class DeweyMap extends SimpleMapModel {    
  HashMap dewey;
 
  DeweyMap() {
    dewey = new HashMap();
  }

  public void addDewey(int d) {
    DataItem item = (DataItem) dewey.get(d);
    if (item == null) {
      item = new DataItem(d);
      dewey.put(d, item);
    }
    item.incrementSize();
    frequency[d]++;
  }

  public void finishAdd() {
    items = new DataItem[dewey.size()];
    dewey.values().toArray(items);
  }
}



//Modified from Sandeep Bhat, MAT259 Winter 2011
public void drawStaticText() {
  float textX, textY;
  textFont(font2, 22); 
  fill(255);
  
  textX = 10;
  textY = 10;
	  
  textAlign(LEFT, CENTER);
  text("Treemap of", textX, textY);
  textY += (textAscent() + textDescent() + 2);
	  
  text("Dewey Categories", textX, textY);
  textY += (textAscent() + textDescent() + 2);
	  
  text("Arts and Recreation (700s)", textX, textY); 
  textY += (textAscent() + textDescent() + 2);
	  
  textFont(font, 14);
  textAlign(LEFT, CENTER);
  textY = height - (textAscent() + textDescent());
  text("Robert Miller, MAT259, Winter 2014 Source: Seattle Public Library", textX, textY);
  textY += (textAscent() + textDescent() + 2);  

}
public void keyReleased() {
  // set layout algorithm
  if (key=='1') layoutAlgorithm = new SquarifiedLayout();
  if (key=='2') layoutAlgorithm = new PivotBySplitSize();
  if (key=='3') layoutAlgorithm = new SliceLayout();
  if (key=='4') layoutAlgorithm = new OrderedTreemap();
  if (key=='5') layoutAlgorithm = new StripTreemap();
  
  //if (key=='a') u = palette[2][random(0,palette[0].length)]);

  if (key=='1'||key=='2'||key=='3'||key=='4'||key=='5'||
    key=='s'||key=='S'||key=='p'||key=='P') loop();
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "sketch_2dspatial" });
  }
}
